'''
mathfun:
submodule for the mathematical functions for TRXASprefitpack

:copyright: 2021 by pistack (Junho Lee).
:license: LGPL3.
'''

from .mathfun import exp_conv_gau, exp_conv_cauchy
from .mathfun import solve_model, compute_model
from .mathfun import compute_signal_gau, compute_signal_cauchy
from .mathfun import compute_signal_pvoigt
from .mathfun import model_n_comp_conv, fact_anal_exp_conv
